import React, { useEffect, useState, useCallback } from 'react';
import Info from '../../components/info';
import { addVisualization } from '../../data-access/store/visualizationSlice';
import { useActiveSaveStateAuthorization, useAppDispatch } from '../../data-access';
import { Visualizations } from '../components/VisualizationPanel';
import { Icon } from '@graphpolaris/shared/lib/components/icon';
import { NodeLinkIcon } from '@graphpolaris/shared/lib/assets/iconVisualizations/NodeLinkIcon';
import { MatrixIcon } from '@graphpolaris/shared/lib/assets/iconVisualizations/MatrixIcon';
import { PaohIcon } from '@graphpolaris/shared/lib/assets/iconVisualizations/PaohIcon';
import { Vis0DIcon } from '@graphpolaris/shared/lib/assets/iconVisualizations/Vis0DIcon';
import { Vis1DIcon } from '@graphpolaris/shared/lib/assets/iconVisualizations/Vis1DIcon';
import { TableIcon } from '@graphpolaris/shared/lib/assets/iconVisualizations/TableIcon';
import { SemSubsIcon } from '@graphpolaris/shared/lib/assets/iconVisualizations/SemSubsIcon';

type VisualizationDescription = {
  name: string;
  displayName: string;
  description: string;
  IconComponent?: React.FC<React.SVGProps<SVGSVGElement>>;
};

const displayNames: { [key: string]: string } = {
  ArcPlot: 'ArcPlot',
  MapVis: 'MapVis',
  MatrixVis: 'MatrixVis',
  NodeLinkVis: 'NodeLinkVis',
  PaohVis: 'PaohVis',
  TableVis: 'TableVis',
  SemanticSubstratesVis: 'SemanticSubstratesVis',
  RawJSONVis: 'RawJSONVis',
  Vis0D: 'Vis0D',
  Vis1D: 'Vis1D',
};

const descriptions: { [key: string]: string } = {
  ArcPlot: '',
  MapVis: 'Geographical Features',
  MatrixVis: 'Overview & Details',
  NodeLinkVis: 'General Patterns and Connections',
  PaohVis: 'Paths and Connection',
  TableVis: 'Node Attribute Statistics and Details',
  SemanticSubstratesVis: 'Node/Edge Attribute Exploration',
  RawJSONVis: '(Raw) Data Export',
  Vis0D: 'KPI visualization',
  Vis1D: 'Basic charts',
};

const iconsVisualization: { [key: string]: React.FC<React.SVGProps<SVGSVGElement>> } = {
  ArcPlot: NodeLinkIcon,
  MapVis: NodeLinkIcon,
  MatrixVis: MatrixIcon,
  NodeLinkVis: NodeLinkIcon,
  PaohVis: PaohIcon,
  TableVis: TableIcon,
  SemanticSubstratesVis: SemSubsIcon,
  RawJSONVis: NodeLinkIcon,
  Vis0D: Vis0DIcon,
  Vis1D: Vis1DIcon,
};

export function Recommender() {
  const dispatch = useAppDispatch();
  const saveStateAuthorization = useActiveSaveStateAuthorization();
  const [visualizationDescriptions, setVisualizationDescriptions] = useState<VisualizationDescription[]>([]);

  useEffect(() => {
    const loadVisualizations = async () => {
      const visDescriptions = await Promise.all(
        Object.keys(Visualizations).map((key) => {
          return {
            name: key,
            displayName: displayNames[key],
            description: descriptions[key],
            IconComponent: iconsVisualization[key],
          };
        }),
      );
      setVisualizationDescriptions(visDescriptions);
    };

    loadVisualizations();
  }, []);

  return (
    <div className="p-4">
      <span className="text-md">Select a visualization</span>
      <div className="grid grid-cols-3 gap-4 my-2">
        {visualizationDescriptions.map(({ name, displayName, description, IconComponent }) => (
          <div
            key={name}
            className={`group flex flex-row item-center p-2 border h-18 ${saveStateAuthorization.visualization.W ? 'cursor-pointer hover:bg-secondary-100' : 'cursor-not-allowed opacity-50'}`}
            onClick={async (e) => {
              e.preventDefault();
              if (!saveStateAuthorization.visualization.W) {
                console.debug('User blocked from editing query due to being a viewer');
                return;
              }

              const component = await Visualizations[name]();
              dispatch(addVisualization({ ...component.default.settings, name: name, id: name }));
            }}
          >
            <div className="m-2 ml-1 mr-1">{IconComponent ? <IconComponent className="h-10 w-auto" /> : null}</div>
            <div className="flex flex-col m-2 w-50">
              <div className="flex items-center justify-between">
                <span className="text-sm font-semibold truncate">{displayName}</span>
              </div>
              <div className="flex items-center justify-between">
                <span className="text-xs text-secondary-400 truncate">{description}</span>
              </div>
            </div>
            <div className="m-2">
              <Info tooltip="Here an explanation" placement="top" />
            </div>
          </div>
        ))}
      </div>
    </div>
  );
}
